% Derivative-Based NSESI approximation (NROSA-DA)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%  Noise-Reduced Reaction-Oriented Sensitivity Analysis  %%%%%%%%%
%%%%%%%%%%            Derivative-Based Approximation              %%%%%%%%%
%%%%%%%%%%               Authors: Zhang/Goutsias                  %%%%%%%%%
%%%%%%%%%%              Last Modified: 09/17/2010                 %%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Initializations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

sigmat(1:M) = 0;
sigmad(1:M) = 0;
sigmas(1:M) = 0;
sigmar(1:M) = 0;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% MC-LHS iterations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Determine LHS permutations
%

for p = 1:P
    perm(p,:) = randperm(L);
end

for iter = 1:L
    
    iterleft = L+1-iter;
    fprintf('Remaining MC-LHS iterations: %6.0f\n',iterleft)
    
    %
    % Calculate Latin hypercube samples
    %
    
    uniform = rand(P,1);
    index   = (perm(:,iter)-1+uniform)./L;
    Z       = repmat(0,[P,1]);
    Z       = sigma*(-sqrt(2)*erfcinv(2*index)); % inverse normal CDF
    svyn    = V*Z;
    kappan  = kappa.*exp(svyn);
    
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Solve system of ODEs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

    xf = repmat(0,[tmax,N,M]);
    xb = repmat(0,[tmax,N,M]);
    for m = 1:M
    
        if (m == 4) || (m == 8) || (m == 16) % Michaelis-Menten kinetics
            kappaf  = kappan;
            kappaf(2*m-1) = kappan(2*m-1).*exp(-delta);
            kappab  = kappan;
            kappab(2*m-1) = kappan(2*m-1).*exp(delta);
        else                                 % Mass Action kinetics
            YR      = zeros(1,M);
            YR(m)   = delta;
            temp    = repmat(YR,[2 1]);      % replicate each element of YR twice
            YR      = temp(:).';             % replicate each element of YR twice
            kappaf  = kappan.*exp(-YR)';
            kappab  = kappan.*exp(+YR)';
        end
    
        if iode == 1
            [t,xf(:,:,m)] = egfr(kappaf,x0,[odetol,odetol],timings);      
            [t,xb(:,:,m)] = egfr(kappab,x0,[odetol,odetol],timings);
        else
            [t,xf(:,:,m)] = ode15s(@netode_EGFR,tpoints,x0,options,kappaf,stoich);
            [t,xb(:,:,m)] = ode15s(@netode_EGFR,tpoints,x0,options,kappab,stoich);
        end
    
    end

    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%% Calculate response characteristics
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %

    for m = 1:M
	    [TTf(m),DDf(m),SSf(m),RRf(m)] = featureit_EGFR(xf(:,:,m),t,tresol,durtol,odetol);
        [TTb(m),DDb(m),SSb(m),RRb(m)] = featureit_EGFR(xb(:,:,m),t,tresol,durtol,odetol);
    end
    
    sigmat = sigmat + ((TTf-TTb)./(2*delta)).^2;
    sigmad = sigmad + ((DDf-DDb)./(2*delta)).^2;
    sigmas = sigmas + ((SSf-SSb)./(2*delta)).^2;
    sigmar = sigmar + ((RRf-RRb)./(2*delta)).^2;
   
end    

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Approximate GSESIs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

sigmat = sigmat./sum(sigmat);
sigmad = sigmad./sum(sigmad);
sigmas = sigmas./sum(sigmas);
sigmar = sigmar./sum(sigmar); 

% Sort SESIs

[setat,IT] = sort(sigmat);
[setad,ID] = sort(sigmad);
[setas,IS] = sort(sigmas);
[setar,IR] = sort(sigmar);